NetworkController = object_add();
object_set_persistent(NetworkController, true);

object_event_add(PlayerControl, ev_step, ev_step_normal, '
	if(not instance_exists(carriers.NetworkController)) {
		instance_create(x,y,carriers.NetworkController);
	}
');

object_event_add(PlayerControl, ev_destroy, 0, '
	with(carriers.NetworkController) {
		instance_destroy();
	}
');

object_event_add(NetworkController, ev_create, 0, '
	sendBuffer = buffer_create();
	if(not global.isHost) {
		with(carriers) {
			if(global.myself.carryEnable)	write_ubyte(other.sendBuffer, CARRY_ENABLE);
			else							write_ubyte(other.sendBuffer, CARRY_DISABLE);
		}
	}
');
object_event_add(NetworkController, ev_destroy, 0, '
	buffer_destroy(sendBuffer);
');
object_event_add(NetworkController, ev_step, ev_step_begin, '
	with(carriers) {
		var readBuffer, sendingPlayer, sendBuffer, read;
		sendBuffer = other.sendBuffer;
		while(PluginPacketGetBuffer(packetID) != -1) {
			readBuffer = PluginPacketGetBuffer(packetID);
			while(buffer_bytes_left(readBuffer) > 0) {
				read = read_ubyte(readBuffer);
				switch(read) {
					case GRAB:
						var grabbedPlayer;
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							grabbedPlayer = execute_string(getCarryable, sendingPlayer);
							if(execute_string(checkGrab, sendingPlayer, grabbedPlayer)) {
								execute_string(sendGrab, sendingPlayer, grabbedPlayer, sendBuffer);
								execute_string(doGrab, sendingPlayer, grabbedPlayer);
							}
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							grabbedPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							execute_string(doGrab, sendingPlayer, grabbedPlayer);
						}
						break;
					case RELEASE:
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							execute_string(sendRelease, sendingPlayer, sendBuffer);
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
						}
						execute_string(doRelease, sendingPlayer);
						break;
					case MODE_RUN:
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							write_ubyte(sendBuffer, MODE_RUN);
							write_ubyte(sendBuffer, ds_list_find_index(global.players, sendingPlayer));
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
						}
						if(sendingPlayer.object != -1)
							sendingPlayer.object.carryRunning = true;
						break;
					case MODE_SHOOT:
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							write_ubyte(sendBuffer, MODE_SHOOT);
							write_ubyte(sendBuffer, ds_list_find_index(global.players, sendingPlayer));
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
						}
						if(sendingPlayer.object != -1)
							sendingPlayer.object.carryRunning = false;
						break;
					case CARRY_ENABLE:
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							sendingPlayer.carryWelcomed = true;
							write_ubyte(sendBuffer, CARRY_ENABLE);
							write_ubyte(sendBuffer, ds_list_find_index(global.players, sendingPlayer));
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
						}
						sendingPlayer.carryEnable = true;
						break;
					case CARRY_DISABLE:
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							sendingPlayer.carryWelcomed = true;
							write_ubyte(sendBuffer, CARRY_DISABLE);
							write_ubyte(sendBuffer, ds_list_find_index(global.players, sendingPlayer));
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
						}
						sendingPlayer.carryEnable = false;
						break;
					case FORBID:
						var otherPlayer;
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							otherPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							execute_string(sendForbid, sendingPlayer, otherPlayer, sendBuffer);
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							otherPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
						}
						execute_string(doForbid, sendingPlayer, otherPlayer);
						break;
					case ALLOW:
						var otherPlayer, otherPlayerIndex;
						if(global.isHost) {
							sendingPlayer = PluginPacketGetPlayer(packetID);
							otherPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							execute_string(sendAllow, sendingPlayer, otherPlayer, sendBuffer);
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							otherPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
						}
						execute_string(doAllow, sendingPlayer, otherPlayer);
						break;
					case BAN:
						if(global.isHost) {
							// shouldnt happen, only host sends
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							execute_string(doBan, sendingPlayer);
						}
						break;
					case UNBAN:
						if(global.isHost) {
							// shouldnt happen, only host sends
						} else {
							sendingPlayer = ds_list_find_value(global.players, read_ubyte(readBuffer));
							execute_string(doUnban, sendingPlayer);
						}
						break;
					case CHAR_SYNC:
						execute_string(deserializeChars, readBuffer);
						break;
					case PLAYER_SYNC:
						execute_string(deserializePlayers, readBuffer);
						break;
					case CONSTS_UPDATE:
						execute_string(receiveUpdateConsts, readBuffer);
						break;
				}
			}
			PluginPacketPop(packetID);
			readBuffer = PluginPacketGetBuffer(packetID);
		}
		if(buffer_size(sendBuffer) > 0) {
			if(global.isHost) {
				execute_string(sendToWelcomedPlayers, sendBuffer);
			} else {
				PluginPacketSend(packetID, sendBuffer);
			}
			buffer_clear(sendBuffer);
		}
	}
');









// SYNC
//
// FULL_UPDATE
//
object_event_add(Player, ev_step, ev_step_end, '
	if(global.isHost) {
		if(carryWelcomed) {
			if(not carryUpdated) {
				carryUpdated = true;
				execute_string(carriers.serializePlayers, carriers.NetworkController.sendBuffer);
				execute_string(carriers.serializeChars, carriers.NetworkController.sendBuffer);
				execute_string(carriers.sendUpdateConsts, carriers.NetworkController.sendBuffer);
			}
		}
	}
');

// QUICK_UPDATE
//
/*object_event_add(PlayerControl, ev_step, ev_step_end, '
	if(global.isHost) {
		if((GameServer.frame mod (10 / global.delta_factor)) == 0) {
			execute_string(carriers.serializeChars, carriers.NetworkController.sendBuffer);
		}
	}
');*/